package Test::Builder2::Formatter::TAP;

use 5.008001;

use Test::Builder2::Mouse;
extends 'Test::Builder2::Formatter::TAP::v13';


=head1 NAME

Test::Builder2::Formatter::TAP - Formatter as the latest stable TAP

=head1 SYNOPSIS

=head1 DESCRIPTION

This library is just a subclass of the latest stable version of TAP.
Currently TAP::v13.

=head1 SEE ALSO

L<< Test::Builder2::Formatter::TAP::v13 >>

=cut

1;
